
#' algdoor

#' algdoor
#' @param service service
#' @param version version
#' @param git_store git_store
#' @param path_store path_store
#' @keywords algoli
#' @export
#' @examples
#' \dontrun{
#' algdoor()
#' }
algdoor <- function(service,version="master",git_store=NULL,path_store=NULL){
  algdoor_object(service=service,version=version,git_store=git_store,path_store=path_store)
}

algdoor_object <- function(service,version,git_store,path_store){

  service_ <- service

  if (!is.null(path_store) || (!is.null(git_store))) {
    if (!is.null(path_store)){

      link_store <- path_store
      service_algoli <- get_service(file.path(link_store,"algoli","service",service,"algoli-service.json"))
      service_functions <- get_functions(service_algoli)
      core_algoli <- get_core(file.path(link_store,"algoli","core",service_algoli$core$name,"algoli-core.json"))
      core_functions <- get_core_functions(core_algoli)

    } else if (!is.null(git_store)){

      link_store <- git_store
      service_algoli <- get_service(file.path(link_store,"algoli","service",service,"-/raw",version,"algoli-service.json"))
      service_functions <- get_functions(service_algoli)
      core_algoli <- get_core(file.path(link_store,"algoli","core",service_algoli$core$name,"-/raw",service_algoli$core$version,"algoli-core.json"))
      core_functions <- get_core_functions(core_algoli)

    }
  } else stop("git_store or path_store required")

  # todo: check every functions before using them

  ###################
  # Core function   #
  ###################

  connect_algdoor <- function(){
    fun <- core_functions$connect
    if (is.null(fun)) {
      stop("algdoor connect function not found")
    } else {
      if (is.null(sc_)){
         sc_ <<- core_functions$connect()
      } else stop("connection done already")
    }
    invisible()
  }

  disconnect_algdoor <- function(){
    fun <- core_functions$disconnect
    if (is.null(fun)) {
      stop("algdoor discconnect function not found")
    } else {
      if (!is.null(sc_)){
         core_functions$disconnect(sc_)
      } else stop("no existing connection")
    }
    invisible()
  }

  df_algdoor <- function(x,name){
    fun <- core_functions$df
    if (is.null(fun)) {
      stop("algdoor df function not found")
    } else {
      res <- core_functions$df(sc=sc_,x=x,name=name)
      return(res)
    }
  }

  copy_algdoor <- function(dataset){
    fun <- core_functions$copy
    if (is.null(fun)) {
      stop("algdoor copy function not found")
    } else {
      res <- core_functions$copy(sc=sc_,dataset=dataset)
      return(res)
    }
  }

  split_algdoor <- function(dataset,train,test,seed){
    fun <- core_functions$split_d
    if (is.null(fun)) {
      stop("algdoor split_d function not found")
    } else {
      res <- core_functions$split_d(dataset,train=train,test=test,seed=seed)
      return(res)
    }
  }

  export_model_algdoor <- function(model,path){
    fun <- core_functions$export_model
    if (is.null(fun)) {
      stop("algdoor export_model function not found")
    } else {
      res <- core_functions$export_model(model=model,path=path)
      return(res)
    }
  }

  load_model_algdoor <- function(path){
    fun <- core_functions$load_model
    if (is.null(fun)) {
      stop("algdoor load_model function not found")
    } else {
      res <- core_functions$load_model(path=path)
      return(res)
    }
  }

  ##########################
  # Fit/predict function   #
  ##########################

  fit_algdoor <- function(algorithm,target,predictors,...){
    if (!is.null(train_)) {
      fun <- service_functions[[paste0(service_,"_fit_",algorithm)]]
      if (is.null(fun)) {
        stop("algdoor fit_algdoor function not found")
      } else {
        res <- fun(target=target,predictors=predictors,train=train_,...)
        return(res)
      }
    } else stop ("Train dataset is missing")

  }

  predict_algdoor <- function(model,newdata=NULL){

    fun <- service_functions[[paste0(service_,"_predict_",model$algorithm)]]
    if (is.null(fun)) {
      stop("algdoor predict_algdoor function not found")
    } else {
      if (!is.null(newdata)){
        if (!is.null(col_bind_)){
          res <- cbind(subset(newdata,select=c(col_bind_)),do.call("fun",list(sc=sc_,model=model,newdata=newdata)) )
        } else {
           res <- fun(sc=sc_,model=model,newdata=newdata)
        }
      } else {
        if (!is.null(col_bind_)){
          res <- cbind(subset(test_,select=c(col_bind_)),do.call("fun",list(sc=sc_,model=model,newdata=test_)) )
        } else {
          res <- fun(sc=sc_,model=model,newdata=test_)
        }
      }
      return(res)
    }
  }

  ###############################
  # pointers to variables in sc #
  ###############################
  sc_ <- NULL
  train_ <- NULL
  test_ <- NULL
  col_bind_ <- NULL


  ######################
  # internal functions #
  ######################
  get_col_bind <- function(col_bind){
    col_bind_ <<- col_bind
    invisible()
  }

  data_train <- function(x){
    train_<<- df_algdoor(x=x,name="train_")
    invisible()
  }

  data_test <- function(x){
    test_  <<- df_algdoor(x=x,name="test_")
    invisible()
  }


  # public
  object <- local({

    connect <- function(){
      connect_algdoor()
      invisible()
    }

    disconnect <- function(){
      disconnect_algdoor()
      invisible()
    }

    dataset <- function(dataset,col_bind=NULL,train=80,test=20,seed=NULL){
      get_col_bind(col_bind)
      tmp <- split_algdoor(copy_algdoor(dataset) ,train=train,test=test,seed=seed)
      if (train!=0L) data_train(x=tmp$train)
      data_test(x=tmp$test)
      invisible()
    }

    fit <- function(algorithm,target,predictors,...){
      res <- fit_algdoor(algorithm=algorithm,target=target,predictors=predictors,...)
      return(res)
    }

    predict <- function(model,newdata=NULL){
      res <- predict_algdoor(model=model, newdata=newdata)
      return(res)
    }

    export_model <- function(model,path){
      res <- export_model_algdoor(model=model,path=path)
      return(res)
    }

    load_model <- function(path){
      res <- load_model_algdoor(path=path)
      return(res)
    }

    summary <- function(){
      cat("Algdoor object\n\n")
      print(service_algoli)
      print(core_algoli)
    }

    environment()
    })
    lockEnvironment(object, TRUE)
    structure(object, class=c("algdoor", class(object)))
}
