#' do_fit

#' do_fit
#' @param service service
#' @param algorithm algorithm in character
#' @param ... other parame related to fitting the model
#' @param dataset tibble or data.frame
#' @param col_bind col_bind in character
#' @param target target in character
#' @param predictors predictors in character
#' @param train train ratio between 0 and 1
#' @param test test ratio between 0 and 1
#' @param version version
#' @param git_store git_store
#' @param path_store path_store
#' @keywords algoli
#' @export
#' @examples
#' \dontrun{
#' do_fit()
#' }
do_fit <- function(service,version="master",algorithm=NULL,...,dataset,col_bind=NULL,target,predictors, train=0.8,test=0.2,path_store=NULL,git_store=NULL){

  # path temporary glow-store to store results
  tmp_path_store <- file.path(tempdir(),"glow-store")

  model_store_path <- file.path(tmp_path_store,"model")
  fileCheck_path <- normalizePath(model_store_path,mustWork = FALSE)
  fileCheck_path <- normalizePath(fileCheck_path,mustWork = FALSE)
  dir.create(fileCheck_path, showWarnings = FALSE, recursive = TRUE)

  # model_id
  id <- stringi::stri_rand_strings(1, 16,'[a-z0-9]')

  fileCheck <- file.path(fileCheck_path,paste0('fit-check_',id,'_.log'))
  file.create(fileCheck)

  target <- gsub("[[:punct:][:blank:]]","_",target)
  predictors <- gsub("[[:punct:][:blank:]]","_",predictors)

  # new algoli
  hello <- algoli::algdoor(service = service,version=version,path_store=path_store,git_store=git_store)

  # Connecting to session
  cat(c(as.character(Sys.time()),"connecting\n"), file=fileCheck,append=TRUE)
  hello$connect()

  # Incoporating dataset into session
  cat(c(as.character(Sys.time()),"copying and splitting dataset\n"), file=fileCheck,append=TRUE)
  hello$dataset(dataset=dataset,col_bind= col_bind, train=train,test=test)

  # Fitting the model on the training dataset
  cat(c(as.character(Sys.time()),"fit model on training\n"), file=fileCheck,append=TRUE)
  res_fit <- hello$fit(algorithm=algorithm,target=target,predictors=predictors,...)

  # exporting model file
  cat(c(as.character(Sys.time()),"export model\n"), file=fileCheck,append=TRUE)
  tmp_model_path <- normalizePath(file.path(model_store_path,id),mustWork = FALSE)
  dir.create(tmp_model_path, showWarnings = TRUE, recursive = TRUE)
  model_path <- hello$export_model(res_fit,tmp_model_path)

  # creating License and metadata model files
  licensing(tmp_model_path) # sofar only MIT + copyright NILU
  meta_model_json(tmp_model_path,
                  algoli_version=as.character(utils::packageVersion("algoli")),
                  model_id=id,
                  path_store = path_store,
                  git_store = git_store,
                  service_name=service,
                  service_version=version,
                  algorithm = algorithm)


  # Prediction on the test dataset ...
  cat(c(as.character(Sys.time()),"predict\n"), file=fileCheck,append=TRUE)
  res_pred <- hello$predict(res_fit)

  res <- list(service = service,
              alg  = algorithm,
              args = list(...),
              target = target,
              predictors = predictors,
              col_bind = col_bind,
              pred = res_pred,
              id = id,
              path_store = tmp_path_store)

  # disconnect ...
  cat(c(as.character(Sys.time()),"disconnect\n"), file=fileCheck,append=TRUE)
  hello$disconnect()

  return(res)
}
