#' do_predict

#' do_predict
#' @param model_id  model id
#' @param version version
#' @param dataset tibble or data.frame
#' @param col_bind col_bind in character
#' @param git_store git_store
#' @param path_store path_store
#' @keywords algoli
#' @export
#' @examples
#' \dontrun{
#' do_predict()
#' }
do_predict <- function(model_id,version="master",dataset=NULL,col_bind=NULL,path_store=NULL,git_store=NULL){

  fileCheck_path <- normalizePath(file.path(tempdir(),"glow-store"),mustWork = FALSE)
  fileCheck_path <- normalizePath(fileCheck_path,mustWork = FALSE)
  dir.create(fileCheck_path, showWarnings = FALSE, recursive = TRUE)

  fileCheck <- file.path(fileCheck_path,paste0('predict-check_',model_id,'_.log'))
  file.create(fileCheck)

  # get model metadata
  if (!is.null(path_store) || (!is.null(git_store))) {
    if (!is.null(path_store)){
      file_path <- file.path(path_store,"model",model_id,"model.json")
    } else if (!is.null(git_store)){
      file_path <- file.path(git_store,"model",model_id,"-/blob",version,"model.json")
    }
  } else stop("git_store or path_store required")

  model_algoli <- jsonlite::fromJSON(file_path, simplifyDataFrame = FALSE)

  # new algoli
  if (!is.null(model_algoli$flow$algoli$path_store)) {
    path_algoli_store <- gsub('//','/',model_algoli$flow$algoli$path_store)
  } else path_algoli_store <- NULL

  if (!is.null(model_algoli$flow$algoli$git_store)) {
    git_algoli_store=gsub('//','/',model_algoli$flow$algoli$git_store)
  } else git_algoli_store <- NULL

  hello <- algoli::algdoor(service = model_algoli$flow$algoli$service$name,version=model_algoli$flow$algoli$service$version,path_store=path_algoli_store,git_store=git_algoli_store)

  # Connecting to session ...
  cat(c(as.character(Sys.time()),"connecting\n"), file=fileCheck,append=TRUE)
  hello$connect()

  # Incoporating dataset into session  ...
  cat(c(as.character(Sys.time()),"copying dataset\n"), file=fileCheck,append=TRUE)
  hello$dataset(dataset=dataset,col_bind= col_bind, train=0L,test=1L)

  # Loading the model ...
  cat(c(as.character(Sys.time()),"loading model\n"), file=fileCheck,append=TRUE)
  # get model binary
  if (!is.null(path_store) || (!is.null(git_store))) {
    if (!is.null(path_store)){
      file_path <- file.path(path_store,"model",model_id,"model.mod")
    } else if (!is.null(git_store)){
      file_path <- file.path(git_store,"model",model_id,"-/blob",version,"model.mod")
    }
  } else stop("git_store or path_store required")

  res_fit <- hello$load_model(file_path)

  # Prediction on the dataset ...
  cat(c(as.character(Sys.time()),"predict\n"), file=fileCheck,append=TRUE)
  res_pred <- hello$predict(res_fit)

  res <- list(pred = res_pred,
              id = model_id,
              path_store=path_store,
              git_store = git_store)

  # disconnect ...
  cat(c(as.character(Sys.time()),"disconnect\n"), file=fileCheck,append=TRUE)
  hello$disconnect()

  return(res)
}
